<?php

namespace ACL\Repositories;

use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\DBAL\Exception;
use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class Operador extends RepositoryBase {

    Use TQuery;

    protected $table = 'OPERADOR';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $organizationId
     * @param string $idoperativoFilter
     * @param null $oAccCustomOrgId
     * @return array
     */
    public function findAllOperador($organizationId, $idoperativoFilter = '', $oAccCustomOrgId = null, $filialFilter = ''): array
    {
        $nrOrgOperQuery = $oAccCustomOrgId ? "'$oAccCustomOrgId'" : "OPER.NRORG";
        $query = <<<SQL
            SELECT * 
              FROM (SELECT DISTINCT OPER.CDOPERADOR, PARC.NRINSCRICAOPARC, PARC.NMPRINCIPALPARC AS NMPARCNEGOCIO, OPER.NMACESSO, 
                           OPER.CDPAIS, OPER.NMOPERADOR, OPER.DTULTATUOPER, OPER.CDCLIENTE, OPER.CDEMPRESA, OPER.DSEMAILOPER, OPER.IDOPERMASTER,
                           OPER.IDOPERACESSO, OPER.IDOPERATIVO, OPER.NRORG, OPER.DTINCLUSAO, OPER.DTULTATU,
                           OPER.NRORGINCLUSAO, OPER.CDOPERINCLUSAO, OPER.CDOPERULTATU, OPER.NRORGULTATU,
                           OPER.NRPARCNEGOCIO, OPER.NRORGSELECIONADA, OPER.CDFILIALSELECIONADA,
                           OPER.NRORGTRAB, OPER.NRESTRUTTRAB, OPER.IDATIVO, OPER.IDTPOPERADOR, ORG.NMORGANIZACAO, EST.NMESTRUTURAM, EST.NRESTRUTURAM,
                           CLI.NMRAZSOCCLIE, PA.NMPAIS, OPER.DTULTACESOPER, V.CDVENDEDOR, V.CDFILIAL, V.CDCAIXA, F.NMFILIAL, C.NMCAIXA,
                           DECODE(OPER.NRORGOPER, NULL, $nrOrgOperQuery, OPER.NRORGOPER) AS NRORGOPER,
                           DECODE(OPER.NRORGOPER, NULL, ORG.NMORGANIZACAO, ORGOPER.NMORGANIZACAO) AS NMORGANIZACAOOPER,
                           (CASE NVL(V.CDVENDEDOR,'NULL') WHEN 'NULL' THEN 0 ELSE 1 END ) AS ADD_SELLER
                      FROM OPERADOR OPER
                 LEFT JOIN ORGANIZACAO ORG ON ORG.NRORG = OPER.NRORG
                 LEFT JOIN ESTRUTURAM EST ON EST.NRORG = OPER.NRORG 
                                         AND EST.NRESTRUTURAM = OPER.NRESTRUTTRAB
                 LEFT JOIN CLIENTE CLI ON CLI.NRORG = OPER.NRORG
                                      AND CLI.CDCLIENTE = OPER.CDCLIENTE
                 LEFT JOIN PAIS PA ON PA.CDPAIS = OPER.CDPAIS
                 LEFT JOIN PARCNEGOCIO PARC ON PARC.NRORG = OPER.NRORG
                                           AND PARC.NRPARCNEGOCIO = OPER.NRPARCNEGOCIO
                 LEFT JOIN VENDEDOR V ON OPER.CDOPERADOR = V.CDOPERADOR AND V.NRORG  = OPER.NRORG
                 LEFT JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL AND V.NRORG  = F.NRORG
                 LEFT JOIN CAIXA C ON V.CDCAIXA = C.CDCAIXA AND V.NRORG  = C.NRORG AND C.CDFILIAL = F.CDFILIAL
                 LEFT JOIN ORGANIZACAO ORGOPER ON ORGOPER.NRORG = OPER.NRORGOPER
                                           )
             WHERE NRORG = $organizationId
             $idoperativoFilter
             $filialFilter
          ORDER BY NMOPERADOR ASC
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    /**
     * @param $nrperfilacesso
     * @param $organizationId
     * @return array
     */
    public function findOperadoresByPerfilAcesso($nrperfilacesso, $organizationId): array
    {
        $query = "SELECT CDOPERADOR, NMOPERADOR, NRPERFACESSPARC, NRORG, NRPARCNEGOCIO,
                         NRPERFILACESSO, IDATIVO, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE, IDOPERATIVO
                    FROM (SELECT O.CDOPERADOR, O.NMOPERADOR, P.NRPERFACESSPARC, P.NRORG, P.NRPARCNEGOCIO,
                                 P.NRPERFILACESSO, P.IDATIVO, P.DTVALIDADE, O.IDOPERATIVO
                            FROM PERFACESSPARC P
                            JOIN OPERADOR O ON (P.NRPARCNEGOCIO = O.NRPARCNEGOCIO AND P.NRORG = O.NRORG)
                         )
                   WHERE NRPERFILACESSO = :NRPERFILACESSO
                     AND NRORG = :NRORG
                     AND IDATIVO = 'S'
                     AND IDOPERATIVO = 'A'";

        $params = array(
            'NRORG' => $organizationId,
            'NRPERFILACESSO' => $nrperfilacesso,
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function findOperadoresForAddPerfilAcesso($nrperfilacesso, $organizationId): array
    {
        $query = "SELECT O.CDOPERADOR, O.NMOPERADOR, P.NRPARCNEGOCIO
                    FROM OPERADOR O
               LEFT JOIN PARCNEGOCIO P ON P.NRPARCNEGOCIO = O.NRPARCNEGOCIO
                   WHERE (P.NRPARCNEGOCIO NOT IN (SELECT NRPARCNEGOCIO 
                                                   FROM PERFACESSPARC
                                                  WHERE NRPERFILACESSO = :NRPERFILACESSO
                                                    AND NRORG = :NRORG) OR O.NRPARCNEGOCIO IS NULL)
                     AND O.NRORG = :NRORG
                     AND O.IDATIVO = 'S'
                     AND O.IDOPERATIVO = 'A'
                ";

        $params = array(
            'NRORG' => $organizationId,
            'NRPERFILACESSO' => $nrperfilacesso,
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    /**
     * @param $cdoperador
     * @param $nrorg
     * @return array[]
     * @throws Exception
     */
    public function findOperadorByCdoperador($cdoperador, $nrorg): array
    {

        $query = "SELECT CDOPERADOR, NMOPERADOR, NRORG
                    FROM OPERADOR
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG = :NRORG";

        $params = array (
            "CDOPERADOR" => $cdoperador,
            "NRORG" => $nrorg
        );

        return InstanceProvider::getEntityManager()->getConnection()->fetchAllAssociative($query, $params);
    }

    /**
     * @param $cdoperador
     * @param $idoperativo
     * @param $idativo
     * @param $nmoperador
     * @param $nmacesso
     * @param $dsemailoper
     * @param $cdpais
     * @param $nrparcnegocio
     * @param $cdcliente
     * @param $nrorg
     * @param $idoperacesso
     * @param $nrorginclusao
     * @param $cdoperinclusao
     * @param $cdsenhaoperweb
     * @param $idopermaster
     * @param $nrestruttrab
     * @param $idtpoperador
     * @param $nrorgoper
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function createOperador(
        $cdoperador, $idoperativo, $idativo, $nmoperador, $nmacesso, $dsemailoper,
        $cdpais, $nrparcnegocio, $cdcliente, $nrorg, $nrorgtrab, $idoperacesso,
        $nrorginclusao, $cdoperinclusao, $cdsenhaoperweb, $idopermaster, $nrestruttrab,
        $idtpoperador, $nrorgoper)
    {
        $query = <<<SQL
            INSERT INTO OPERADOR
                        (CDOPERADOR, IDOPERATIVO, IDATIVO, NMOPERADOR, NMACESSO, DSEMAILOPER, CDPAIS, CDSENHAOPERWEB, NRPARCNEGOCIO, IDOPERMASTER,
                        CDCLIENTE, NRORG, NRORGTRAB, IDOPERACESSO, NRORGINCLUSAO,
                        CDOPERINCLUSAO, DTINCLUSAO, DTULTATUOPER, NRESTRUTTRAB, IDTPOPERADOR, NRORGOPER)
                 VALUES (:CDOPERADOR, :IDOPERATIVO, :IDATIVO, :NMOPERADOR, :NMACESSO, :DSEMAILOPER, :CDPAIS, :CDSENHAOPERWEB,
                         :NRPARCNEGOCIO, :IDOPERMASTER, :CDCLIENTE, :NRORG, :NRORGTRAB, :IDOPERACESSO,
                         :NRORGINCLUSAO, :CDOPERINCLUSAO, SYSDATE, SYSDATE, :NRESTRUTTRAB, :IDTPOPERADOR, :NRORGOPER)
SQL;

        $params = array (
            "CDOPERADOR"          => $cdoperador,
            "IDOPERATIVO"         => $idoperativo,
            "IDATIVO"             => $idativo,
            "NMOPERADOR"          => $nmoperador,
            "NMACESSO"            => $nmacesso,
            "DSEMAILOPER"         => $dsemailoper,
            "CDPAIS"              => $cdpais,
            "NRPARCNEGOCIO"       => $nrparcnegocio,
            "CDCLIENTE"           => $cdcliente,
            "NRORG"               => $nrorg,
            "NRORGTRAB"           => $nrorgtrab,
            "IDOPERACESSO"        => $idoperacesso,
            "NRORGINCLUSAO"       => $nrorginclusao,
            "CDOPERINCLUSAO"      => $cdoperinclusao,
            "CDSENHAOPERWEB"      => $cdsenhaoperweb,
            "IDOPERMASTER"        => $idopermaster,
            "NRESTRUTTRAB"        => $nrestruttrab,
            "IDTPOPERADOR"        => $idtpoperador,
            "NRORGOPER"           => $nrorgoper
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    /**
     * @param $nrparcnegocio
     * @param $cdoperador
     * @param $nrorg
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function updateParcnegocioInOperador($nrparcnegocio, $cdoperador, $nrorg)
    {
        $query = "UPDATE OPERADOR
                     SET NRPARCNEGOCIO = :NRPARCNEGOCIO
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG = :NRORG";

        $params = array (
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "CDOPERADOR"    => $cdoperador,
            "NRORG"         => $nrorg
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function editOperator(
        $cdoperador, $idoperativo, $idativo, $nmoperador, $nmacesso, $dsemailope,
        $cdpais, $cdcliente, $nrorg, $cdoperultatu, $nrparcnegocio,
        $idopermaster, $nrestruttrab, $idtpoperador, $nrorgoper)
    {
        $query = "UPDATE OPERADOR
                     SET IDOPERATIVO     = :IDOPERATIVO,
                         IDATIVO         = :IDATIVO,
                         NMOPERADOR      = :NMOPERADOR,
                         NMACESSO        = :NMACESSO,
                         DSEMAILOPER     = :DSEMAILOPER,
                         CDPAIS          = :CDPAIS,
                         CDCLIENTE       = :CDCLIENTE,
                         CDOPERULTATU    = :CDOPERULTATU,
                         NRORGULTATU     = :NRORG,
                         DTULTATUOPER    = SYSDATE,
                         NRPARCNEGOCIO   = :NRPARCNEGOCIO,
                         IDOPERMASTER    = :IDOPERMASTER,
                         NRESTRUTTRAB    = :NRESTRUTTRAB,
                         IDTPOPERADOR    = :IDTPOPERADOR,
                         NRORGOPER       = :NRORGOPER
                   WHERE CDOPERADOR      = :CDOPERADOR
                     AND NRORG           = :NRORG";

        $params = array (
            "IDOPERATIVO"   => $idoperativo,
            "IDATIVO"       => $idativo,
            "NMOPERADOR"    => $nmoperador,
            "NMACESSO"      => $nmacesso,
            "DSEMAILOPER"   => $dsemailope,
            "CDPAIS"        => $cdpais,
            "CDCLIENTE"     => $cdcliente,
            "NRORG"         => $nrorg,
            "CDOPERULTATU"  => $cdoperultatu,
            "CDOPERADOR"    => $cdoperador,
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "IDOPERMASTER"  => $idopermaster,
            "NRESTRUTTRAB"  => $nrestruttrab,
            "IDTPOPERADOR"  => $idtpoperador,
            "NRORGOPER"     => $nrorgoper
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    /**
     * @param $passwordWeb
     * @param $passwordDelphi
     * @param $cdoperador
     * @param $nrorg
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Exception
     */
    public function resetPassword($passwordWeb, $passwordDelphi, $cdoperador, $nrorg)
    {
        $query = "UPDATE OPERADOR
                     SET CDSENHAOPERWEB = :CDSENHAOPERWEB,
                         CDSENHOPER     = :CDSENHOPER,
                         IDSTATUSENHA   = 'A'
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG       = :NRORG";

        $params = array(
            'CDSENHAOPERWEB' => $passwordWeb,
            'CDSENHOPER'     => $passwordDelphi,
            'CDOPERADOR'     => $cdoperador,
            'NRORG'          => $nrorg
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->executeQuery($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function findLogOpera($nrorg): array
    {
        $query = "SELECT DISTINCT NVL(NMRESPOPER, NMOPERADOR) AS NMRESPOPER, NMTABELA, 
                    DTOPERACAO, IDTIPOOPER, DSOPERACAO, CDPESQITEM, CDFILLOGOPER, NMFILIAL
                  FROM LOGOPERA L
                  LEFT JOIN FILIAL F   ON L.CDFILLOGOPER = F.CDFILIAL
                  LEFT JOIN OPERADOR O ON L.CDORESPOPER  = O.CDOPERADOR
                  UNION ALL
                  SELECT DISTINCT NVL(NMRESPOPER, NMOPERADOR) AS NMRESPOPER, NMTABELA, 
                    DTOPERACAO, IDTIPOOPER, DSOPERACAO, CDPESQITEM, FO.CDFILIAL AS CDFILLOGOPER, NMFILIAL
                  FROM LOGREGRAFISCAL L
                  LEFT JOIN OPERADOR O  ON L.CDORESPOPER = O.CDOPERADOR
                  LEFT JOIN FILIOPER FO ON FO.CDOPERADOR = O.CDOPERADOR
                  LEFT JOIN FILIAL F    ON FO.CDFILIAL   = F.CDFILIAL
                  UNION ALL
                  SELECT DISTINCT NMRESPOPERVND AS NMRESPOPER, NMTABELAVND AS NMTABELA, DTOPERACAOVND AS DTOPERACAO, IDTIPOOPERVND AS IDTIPOOPER, 
                        DSOPERACAOVND AS DSOPERACAO, CDPESQITEMVND AS CDPESQITEM, CDFILLOGOPERVND AS CDFILLOGOPER, NMFILIAL
                  FROM LOGOPERAVND L
                  LEFT JOIN FILIAL F ON L.CDFILLOGOPERVND = F.CDFILIAL";

        $params = array (
            "NRORG" => $nrorg
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }
}
