<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Repositories\Opergrupop as OpergrupopRepository;
use Exception;
use Teknisa\Libs\Util\Environment;

class Opergrupop
{

    /**
     * @var Environment
     */
    private $libEnvironment;

    /**
     * @var OpergrupopRepository
     */
    private $opergrupopRepository;

    public function __construct(OpergrupopRepository $opergrupopRepository)
    {
        $this->opergrupopRepository = $opergrupopRepository;
        $this->libEnvironment =  InstanceProvider::getLibEnvironment();
    }

    /**
     * @param $cdoperador
     * @param $sqlOperator
     * @return array
     */
    public function findOpergrupopByOperador($cdoperador, $sqlOperator): array
    {
        if ($sqlOperator == 'IN') {
            return $this->opergrupopRepository->findOpergrupopByOperador($cdoperador, getNrorg());
        }
        return $this->opergrupopRepository->findGrupoperNotInOpergrupop($cdoperador, getNrorg());
    }

    /**
     * @param $params
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function createOpergrupop($params)
    {
        foreach ($params as $row) {
            $this->opergrupopRepository->createOpergrupop(
                $row['CDGRUPOPER'], $row['CDOPERADOR'], 'N', getNrorg(), getCurrentUserId()
            );
        }
    }

    /**
     * @param $cdgrupoper
     * @param $cdoperador
     * @param $idmenuprinop
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function updateOpergrupop($cdgrupoper, $cdoperador, $idmenuprinop)
    {
        $cdgrupoper = (string)$cdgrupoper;
        if (strlen($cdgrupoper) < 2) {
            $cdgrupoper = str_pad($cdgrupoper, 2, '0', STR_PAD_LEFT);
        }

        if ($idmenuprinop == 'S') {
            $opergrupop = $this->opergrupopRepository->findOpergrupopByOperador($cdoperador, getNrorg());
            foreach ($opergrupop['data'] as $row) {
                if ($row['CDGRUPOPER'] === $cdgrupoper) {
                    $this->opergrupopRepository->updateOpergrupop($row['CDGRUPOPER'], $row['CDOPERADOR'], 'S');
                } else if ($row['IDMENUPRINOP'] == 'S') {
                    $this->opergrupopRepository->updateOpergrupop($row['CDGRUPOPER'], $row['CDOPERADOR'], 'N');
                }
            }
        } else {
            $this->opergrupopRepository->updateOpergrupop($cdgrupoper, $cdoperador, 'N');
        }
    }

    /**
     * @param $params
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function deleteOpergrupop($params)
    {
        foreach ($params as $row) {
            $this->opergrupopRepository->daleteOpergrupop($row['CDGRUPOPER'], $row['CDOPERADOR'], getNrorg());
        }
    }
}
